/* Copyright (C) 2017 Mono Wireless Inc. All Rights Reserved.    *
 * Released under MW-SLA-*J,*E (MONO WIRELESS SOFTWARE LICENSE   *
 * AGREEMENT).                                                   */

/*
 * 本コードはTWELITE標準シリアル電文形式のパーサの簡単な使用例です。
 */

#include <SoftwareSerial.h>
#include <sercmd.h>

String VersionNumber = "0.1.0";

// Prototype
void MWPutChar( unsigned char chr );

// Global Variables
SoftwareSerial MWSerial(2, 3); // RX, TX
sercmd Parse(false, 0, MWPutChar );     // パーサのオブジェクト

void MWPutChar( unsigned char chr )
{
  Serial.print((char)chr);
}

void setup() {
  Serial.begin(38400);
  MWSerial.begin(38400);
  Serial.print("MONOWIRELESS Serial Parser v");
  Serial.println(VersionNumber);
  Serial.println();
}

void loop() {
  // put your main code here, to run repeatedly:
  int c;
  c = MWSerial.read();
  if(c >= 0){
    byte u8stat;
    // １バイトずつ解釈をしていく
    u8stat = Parse.u8Parse(c);

    // 解釈に成功したら終了
    if (u8stat == E_SERCMD_COMPLETE) {
      // 系列を読み出せたので表示
      Serial.print("u8Parse() success, bytes=");
      Serial.print( Parse.u16len ); // データの長さは u16len
      Serial.print(", payload=");

      for(int j=0; j< Parse.u16len; j++) {
        if(Parse.au8data[j] < 0x10) Serial.print(0);
        Serial.print(Parse.au8data[j], HEX);  // ペイロードは au8data
      }
      Serial.println();

      // 入力した系列をそのまま vOutput で出力する
      //   出力先は vPutChar コマンドにて
      Serial.print("vOutput() -> ");
      Parse.vOutput();
      Serial.println();
    } else if (u8stat == E_SERCMD_CHECKSUM_ERROR) {
      // 電文中のチェックサムと計算したチェックサムが異なっているので、おそらく正しく読み込めなかった
      Serial.print("Command checksum error\n");
    }
  }
}
